package cn.ibizlab.ehr.core.att.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.att.domain.AttEndenceCalendar;
import cn.ibizlab.ehr.core.att.filter.AttEndenceCalendarSearchContext;
import cn.ibizlab.ehr.core.att.service.IAttEndenceCalendarService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.att.mapper.AttEndenceCalendarMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[考勤日历] 服务对象接口实现
 */
@Slf4j
@Service("AttEndenceCalendarServiceImpl")
public class AttEndenceCalendarServiceImpl extends ServiceImpl<AttEndenceCalendarMapper, AttEndenceCalendar> implements IAttEndenceCalendarService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmOrgService ormorgService;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.vac.service.IVacHolidayService vacholidayService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(AttEndenceCalendar et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getAttendencecalendarid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<AttEndenceCalendar> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(AttEndenceCalendar et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("attendencecalendarid", et.getAttendencecalendarid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getAttendencecalendarid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<AttEndenceCalendar> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public AttEndenceCalendar get(String key) {
        AttEndenceCalendar et = getById(key);
        if(et == null){
            et = new AttEndenceCalendar();
            et.setAttendencecalendarid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public AttEndenceCalendar getDraft(AttEndenceCalendar et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(AttEndenceCalendar et) {
        return (!ObjectUtils.isEmpty(et.getAttendencecalendarid())) && (!Objects.isNull(this.getById(et.getAttendencecalendarid())));
    }
    @Override
    @Transactional
    public boolean save(AttEndenceCalendar et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(AttEndenceCalendar et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<AttEndenceCalendar> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<AttEndenceCalendar> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<AttEndenceCalendar> selectByOrmorgid(String orgid) {
        return baseMapper.selectByOrmorgid(orgid);
    }
    @Override
    public void removeByOrmorgid(String orgid) {
        this.remove(new QueryWrapper<AttEndenceCalendar>().eq("ormorgid",orgid));
    }

	@Override
    public List<AttEndenceCalendar> selectByVacholidayid(String vacholidayid) {
        return baseMapper.selectByVacholidayid(vacholidayid);
    }
    @Override
    public void removeByVacholidayid(Collection<String> ids) {
        this.remove(new QueryWrapper<AttEndenceCalendar>().in("vacholidayid",ids));
    }

    @Override
    public void removeByVacholidayid(String vacholidayid) {
        this.remove(new QueryWrapper<AttEndenceCalendar>().eq("vacholidayid",vacholidayid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<AttEndenceCalendar> searchDefault(AttEndenceCalendarSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<AttEndenceCalendar> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<AttEndenceCalendar>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(AttEndenceCalendar et){
        //实体关系[DER1N_ATTENDENCECALENDAR_ORMORG_ORMORGID]
        if(!ObjectUtils.isEmpty(et.getOrmorgid())){
            cn.ibizlab.ehr.core.orm.domain.OrmOrg ormorg=et.getOrmorg();
            if(ObjectUtils.isEmpty(ormorg)){
                cn.ibizlab.ehr.core.orm.domain.OrmOrg majorEntity=ormorgService.get(et.getOrmorgid());
                et.setOrmorg(majorEntity);
                ormorg=majorEntity;
            }
            et.setZzdzs(ormorg.getZzdzs());
            et.setOrmorgname(ormorg.getOrgname());
        }
        //实体关系[DER1N_ATTENDENCECALENDAR_VACHOLIDAY_VACHOLIDAYID]
        if(!ObjectUtils.isEmpty(et.getVacholidayid())){
            cn.ibizlab.ehr.core.vac.domain.VacHoliday vacholiday=et.getVacholiday();
            if(ObjectUtils.isEmpty(vacholiday)){
                cn.ibizlab.ehr.core.vac.domain.VacHoliday majorEntity=vacholidayService.get(et.getVacholidayid());
                et.setVacholiday(majorEntity);
                vacholiday=majorEntity;
            }
            et.setJjrlx(vacholiday.getJjrlx());
            et.setVacholidayname(vacholiday.getVacholidayname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<AttEndenceCalendar> getAttendencecalendarByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<AttEndenceCalendar> getAttendencecalendarByEntities(List<AttEndenceCalendar> entities) {
        List ids =new ArrayList();
        for(AttEndenceCalendar entity : entities){
            Serializable id=entity.getAttendencecalendarid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



