package cn.ibizlab.ehr.core.common.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.common.domain.SysAdminFunc;
import cn.ibizlab.ehr.core.common.filter.SysAdminFuncSearchContext;
import cn.ibizlab.ehr.core.common.service.ISysAdminFuncService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.common.mapper.SysAdminFuncMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[系统管理功能] 服务对象接口实现
 */
@Slf4j
@Service("SysAdminFuncServiceImpl")
public class SysAdminFuncServiceImpl extends ServiceImpl<SysAdminFuncMapper, SysAdminFunc> implements ISysAdminFuncService {

    @Autowired
    @Lazy
    private cn.ibizlab.ehr.core.common.service.ISysAdminService sysadminService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(SysAdminFunc et) {
        fillParentData(et);
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("sysadminfuncid",et.getSysadminfuncid())))
            return false;
        CachedBeanCopier.copy(get(et.getSysadminfuncid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<SysAdminFunc> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(SysAdminFunc et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(SysAdminFunc et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<SysAdminFunc> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    public SysAdminFunc getDraft(SysAdminFunc et) {
        fillParentData(et);
        return et;
    }

    @Override
    @Transactional
    public boolean create(SysAdminFunc et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getSysadminfuncid()),et);
        return true;
    }

    @Override
    public void createBatch(List<SysAdminFunc> list) {
        list.forEach(item->fillParentData(item));
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public SysAdminFunc get(String key) {
        SysAdminFunc et = getById(key);
        if(et==null){
            et=new SysAdminFunc();
            et.setSysadminfuncid(key);
        }
        else{
        }
        return et;
    }

    @Override
    public boolean checkKey(SysAdminFunc et) {
        return (!ObjectUtils.isEmpty(et.getSysadminfuncid()))&&(!Objects.isNull(this.getById(et.getSysadminfuncid())));
    }


	@Override
    public List<SysAdminFunc> selectBySysadminid(String sysadminid) {
        return baseMapper.selectBySysadminid(sysadminid);
    }

    @Override
    public void removeBySysadminid(String sysadminid) {
        this.remove(new QueryWrapper<SysAdminFunc>().eq("sysadminid",sysadminid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SysAdminFunc> searchDefault(SysAdminFuncSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SysAdminFunc> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SysAdminFunc>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(SysAdminFunc et){
        //实体关系[DER1N_SYSADMINFUNC_SYSADMIN_SYSADMINID]
        if(!ObjectUtils.isEmpty(et.getSysadminid())){
            cn.ibizlab.ehr.core.common.domain.SysAdmin sysadmin=et.getSysadmin();
            if(ObjectUtils.isEmpty(sysadmin)){
                cn.ibizlab.ehr.core.common.domain.SysAdmin majorEntity=sysadminService.get(et.getSysadminid());
                et.setSysadmin(majorEntity);
                sysadmin=majorEntity;
            }
            et.setSysadminname(sysadmin.getSysadminname());
            et.setAdminobject(sysadmin.getAdminobject());
        }
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


