package cn.ibizlab.ehr.core.orm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.orm.domain.OrmErporg;
import cn.ibizlab.ehr.core.orm.filter.OrmErporgSearchContext;
import cn.ibizlab.ehr.core.orm.service.IOrmErporgService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.orm.mapper.OrmErporgMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[ERP组织中间表] 服务对象接口实现
 */
@Slf4j
@Service("OrmErporgServiceImpl")
public class OrmErporgServiceImpl extends ServiceImpl<OrmErporgMapper, OrmErporg> implements IOrmErporgService {


    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(OrmErporg et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getOrmerporgid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<OrmErporg> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(OrmErporg et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("ormerporgid", et.getOrmerporgid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getOrmerporgid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<OrmErporg> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public OrmErporg get(String key) {
        OrmErporg et = getById(key);
        if(et == null){
            et = new OrmErporg();
            et.setOrmerporgid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public OrmErporg getDraft(OrmErporg et) {
        return et;
    }

    @Override
    public boolean checkKey(OrmErporg et) {
        return (!ObjectUtils.isEmpty(et.getOrmerporgid())) && (!Objects.isNull(this.getById(et.getOrmerporgid())));
    }
    @Override
    @Transactional
    public boolean save(OrmErporg et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(OrmErporg et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<OrmErporg> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<OrmErporg> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    @Transactional
    public OrmErporg synOrg(OrmErporg et) {
        //自定义代码
        return et;
    }



    /**
     * 查询集合 查询可用ERP组织
     */
    @Override
    public Page<OrmErporg> searchCX(OrmErporgSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<OrmErporg> pages=baseMapper.searchCX(context.getPages(),context,context.getSelectCond());
        return new PageImpl<OrmErporg>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 查询可用ERP部门
     */
    @Override
    public Page<OrmErporg> searchCXBM(OrmErporgSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<OrmErporg> pages=baseMapper.searchCXBM(context.getPages(),context,context.getSelectCond());
        return new PageImpl<OrmErporg>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 查询可用ERP部门
     */
    @Override
    public Page<OrmErporg> searchCXZBM(OrmErporgSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<OrmErporg> pages=baseMapper.searchCXZBM(context.getPages(),context,context.getSelectCond());
        return new PageImpl<OrmErporg>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<OrmErporg> searchDefault(OrmErporgSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<OrmErporg> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<OrmErporg>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 管理单位选择
     */
    @Override
    public Page<OrmErporg> searchLegalChoice(OrmErporgSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<OrmErporg> pages=baseMapper.searchLegalChoice(context.getPages(),context,context.getSelectCond());
        return new PageImpl<OrmErporg>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<OrmErporg> getOrmerporgByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<OrmErporg> getOrmerporgByEntities(List<OrmErporg> entities) {
        List ids =new ArrayList();
        for(OrmErporg entity : entities){
            Serializable id=entity.getOrmerporgid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



