package cn.ibizlab.ehr.core.orm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.orm.domain.OrmUser;
import cn.ibizlab.ehr.core.orm.filter.OrmUserSearchContext;
import cn.ibizlab.ehr.core.orm.service.IOrmUserService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.orm.mapper.OrmUserMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[用户管理] 服务对象接口实现
 */
@Slf4j
@Service("OrmUserServiceImpl")
public class OrmUserServiceImpl extends ServiceImpl<OrmUserMapper, OrmUser> implements IOrmUserService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmOrgsectorService ormorgsectorService;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmOrgService ormorgService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(OrmUser et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getOrguserid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<OrmUser> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(OrmUser et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("orguserid", et.getOrguserid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getOrguserid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<OrmUser> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public OrmUser get(String key) {
        OrmUser et = getById(key);
        if(et == null){
            et = new OrmUser();
            et.setOrguserid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public OrmUser getDraft(OrmUser et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(OrmUser et) {
        return (!ObjectUtils.isEmpty(et.getOrguserid())) && (!Objects.isNull(this.getById(et.getOrguserid())));
    }
    @Override
    @Transactional
    public boolean save(OrmUser et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(OrmUser et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<OrmUser> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<OrmUser> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<OrmUser> selectByOrgsectorid(String orgsectorid) {
        return baseMapper.selectByOrgsectorid(orgsectorid);
    }
    @Override
    public List<OrmUser> selectByOrgsectorid(Collection<String> ids) {
        return this.list(new QueryWrapper<OrmUser>().in("orgsectorid",ids));
    }

    @Override
    public void removeByOrgsectorid(String orgsectorid) {
        this.remove(new QueryWrapper<OrmUser>().eq("orgsectorid",orgsectorid));
    }

	@Override
    public List<OrmUser> selectByOrgid(String orgid) {
        return baseMapper.selectByOrgid(orgid);
    }
    @Override
    public List<OrmUser> selectByOrgid(Collection<String> ids) {
        return this.list(new QueryWrapper<OrmUser>().in("orgid",ids));
    }

    @Override
    public void removeByOrgid(String orgid) {
        this.remove(new QueryWrapper<OrmUser>().eq("orgid",orgid));
    }


    /**
     * 查询集合 当前组织及下级组织
     */
    @Override
    public Page<OrmUser> searchDQZZJXJZZ(OrmUserSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<OrmUser> pages=baseMapper.searchDQZZJXJZZ(context.getPages(),context,context.getSelectCond());
        return new PageImpl<OrmUser>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<OrmUser> searchDefault(OrmUserSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<OrmUser> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<OrmUser>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(OrmUser et){
        //实体关系[DER1N_ORMUSER_ORMORGSECTOR_ORGSECTORID]
        if(!ObjectUtils.isEmpty(et.getOrgsectorid())){
            cn.ibizlab.ehr.core.orm.domain.OrmOrgsector ormorgsector=et.getOrmorgsector();
            if(ObjectUtils.isEmpty(ormorgsector)){
                cn.ibizlab.ehr.core.orm.domain.OrmOrgsector majorEntity=ormorgsectorService.get(et.getOrgsectorid());
                et.setOrmorgsector(majorEntity);
                ormorgsector=majorEntity;
            }
            et.setOrgsectorname(ormorgsector.getOrgsectorname());
        }
        //实体关系[DER1N_ORMUSER_ORMORG_ORGID]
        if(!ObjectUtils.isEmpty(et.getOrgid())){
            cn.ibizlab.ehr.core.orm.domain.OrmOrg org=et.getOrg();
            if(ObjectUtils.isEmpty(org)){
                cn.ibizlab.ehr.core.orm.domain.OrmOrg majorEntity=ormorgService.get(et.getOrgid());
                et.setOrg(majorEntity);
                org=majorEntity;
            }
            et.setOrgname(org.getOrgname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<OrmUser> getOrmuserByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<OrmUser> getOrmuserByEntities(List<OrmUser> entities) {
        List ids =new ArrayList();
        for(OrmUser entity : entities){
            Serializable id=entity.getOrguserid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



