package cn.ibizlab.ehr.core.par.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.par.domain.ParJxzgpcmx;
import cn.ibizlab.ehr.core.par.filter.ParJxzgpcmxSearchContext;
import cn.ibizlab.ehr.core.par.service.IParJxzgpcmxService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.par.mapper.ParJxzgpcmxMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[绩效主观评测明细] 服务对象接口实现
 */
@Slf4j
@Service("ParJxzgpcmxServiceImpl")
public class ParJxzgpcmxServiceImpl extends ServiceImpl<ParJxzgpcmxMapper, ParJxzgpcmx> implements IParJxzgpcmxService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.par.service.IParJxzgpcService parjxzgpcService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(ParJxzgpcmx et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getParjxzgpcmxid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<ParJxzgpcmx> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(ParJxzgpcmx et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("parjxzgpcmxid", et.getParjxzgpcmxid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getParjxzgpcmxid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<ParJxzgpcmx> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public ParJxzgpcmx get(String key) {
        ParJxzgpcmx et = getById(key);
        if(et == null){
            et = new ParJxzgpcmx();
            et.setParjxzgpcmxid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public ParJxzgpcmx getDraft(ParJxzgpcmx et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(ParJxzgpcmx et) {
        return (!ObjectUtils.isEmpty(et.getParjxzgpcmxid())) && (!Objects.isNull(this.getById(et.getParjxzgpcmxid())));
    }
    @Override
    @Transactional
    public boolean save(ParJxzgpcmx et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(ParJxzgpcmx et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<ParJxzgpcmx> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<ParJxzgpcmx> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<ParJxzgpcmx> selectByParjxzgpcid(String parjxzgpcid) {
        return baseMapper.selectByParjxzgpcid(parjxzgpcid);
    }
    @Override
    public void resetByParjxzgpcid(String parjxzgpcid) {
        this.update(new UpdateWrapper<ParJxzgpcmx>().set("parjxzgpcid",null).eq("parjxzgpcid",parjxzgpcid));
    }

    @Override
    public void resetByParjxzgpcid(Collection<String> ids) {
        this.update(new UpdateWrapper<ParJxzgpcmx>().set("parjxzgpcid",null).in("parjxzgpcid",ids));
    }

    @Override
    public void removeByParjxzgpcid(String parjxzgpcid) {
        this.remove(new QueryWrapper<ParJxzgpcmx>().eq("parjxzgpcid",parjxzgpcid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<ParJxzgpcmx> searchDefault(ParJxzgpcmxSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<ParJxzgpcmx> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<ParJxzgpcmx>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(ParJxzgpcmx et){
        //实体关系[DER1N_PARJXZGPCMX_PARJXZGPC_PARJXZGPCID]
        if(!ObjectUtils.isEmpty(et.getParjxzgpcid())){
            cn.ibizlab.ehr.core.par.domain.ParJxzgpc parjxzgpc=et.getParjxzgpc();
            if(ObjectUtils.isEmpty(parjxzgpc)){
                cn.ibizlab.ehr.core.par.domain.ParJxzgpc majorEntity=parjxzgpcService.get(et.getParjxzgpcid());
                et.setParjxzgpc(majorEntity);
                parjxzgpc=majorEntity;
            }
            et.setParjxzgpcname(parjxzgpc.getParjxzgpcname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<ParJxzgpcmx> getParjxzgpcmxByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<ParJxzgpcmx> getParjxzgpcmxByEntities(List<ParJxzgpcmx> entities) {
        List ids =new ArrayList();
        for(ParJxzgpcmx entity : entities){
            Serializable id=entity.getParjxzgpcmxid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



