package cn.ibizlab.ehr.core.pcm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.pcm.domain.PcmBdsqd;
import cn.ibizlab.ehr.core.pcm.filter.PcmBdsqdSearchContext;
import cn.ibizlab.ehr.core.pcm.service.IPcmBdsqdService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.pcm.mapper.PcmBdsqdMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[变动申请单（停用）] 服务对象接口实现
 */
@Slf4j
@Service("PcmBdsqdServiceImpl")
public class PcmBdsqdServiceImpl extends ServiceImpl<PcmBdsqdMapper, PcmBdsqd> implements IPcmBdsqdService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pcm.service.IPcmBdsqdmxService pcmbdsqdmxService;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pim.service.IPimPersonService pimpersonService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(PcmBdsqd et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmbdsqdid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<PcmBdsqd> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(PcmBdsqd et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("pcmbdsqdid", et.getPcmbdsqdid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmbdsqdid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<PcmBdsqd> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        pcmbdsqdmxService.resetByPcmbdsqdid(key);
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        pcmbdsqdmxService.resetByPcmbdsqdid(idList);
        removeByIds(idList);
    }

    @Override
    @Transactional
    public PcmBdsqd get(String key) {
        PcmBdsqd et = getById(key);
        if(et == null){
            et = new PcmBdsqd();
            et.setPcmbdsqdid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public PcmBdsqd getDraft(PcmBdsqd et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(PcmBdsqd et) {
        return (!ObjectUtils.isEmpty(et.getPcmbdsqdid())) && (!Objects.isNull(this.getById(et.getPcmbdsqdid())));
    }
    @Override
    @Transactional
    public boolean save(PcmBdsqd et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(PcmBdsqd et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<PcmBdsqd> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<PcmBdsqd> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<PcmBdsqd> selectByPimpersonid(String pimpersonid) {
        return baseMapper.selectByPimpersonid(pimpersonid);
    }
    @Override
    public void resetByPimpersonid(String pimpersonid) {
        this.update(new UpdateWrapper<PcmBdsqd>().set("pimpersonid",null).eq("pimpersonid",pimpersonid));
    }

    @Override
    public void resetByPimpersonid(Collection<String> ids) {
        this.update(new UpdateWrapper<PcmBdsqd>().set("pimpersonid",null).in("pimpersonid",ids));
    }

    @Override
    public void removeByPimpersonid(String pimpersonid) {
        this.remove(new QueryWrapper<PcmBdsqd>().eq("pimpersonid",pimpersonid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PcmBdsqd> searchDefault(PcmBdsqdSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmBdsqd> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmBdsqd>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(PcmBdsqd et){
        //实体关系[DER1N_PCMBDSQD_PIMPERSON_PIMPERSONID]
        if(!ObjectUtils.isEmpty(et.getPimpersonid())){
            cn.ibizlab.ehr.core.pim.domain.PimPerson pimperson=et.getPimperson();
            if(ObjectUtils.isEmpty(pimperson)){
                cn.ibizlab.ehr.core.pim.domain.PimPerson majorEntity=pimpersonService.get(et.getPimpersonid());
                et.setPimperson(majorEntity);
                pimperson=majorEntity;
            }
            et.setPimpersonname(pimperson.getPimpersonname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PcmBdsqd> getPcmbdsqdByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PcmBdsqd> getPcmbdsqdByEntities(List<PcmBdsqd> entities) {
        List ids =new ArrayList();
        for(PcmBdsqd entity : entities){
            Serializable id=entity.getPcmbdsqdid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



