package cn.ibizlab.ehr.core.pcm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.pcm.domain.PcmGxml;
import cn.ibizlab.ehr.core.pcm.filter.PcmGxmlSearchContext;
import cn.ibizlab.ehr.core.pcm.service.IPcmGxmlService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.pcm.mapper.PcmGxmlMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[高校名录] 服务对象接口实现
 */
@Slf4j
@Service("PcmGxmlServiceImpl")
public class PcmGxmlServiceImpl extends ServiceImpl<PcmGxmlMapper, PcmGxml> implements IPcmGxmlService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pcm.service.IPCMGxxkTempService pcmgxxktempService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(PcmGxml et) {
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmgxmlid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<PcmGxml> list) {
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(PcmGxml et) {
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("pcmgxmlid", et.getPcmgxmlid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmgxmlid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<PcmGxml> list) {
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        pcmgxxktempService.removeByPcmgxmlid(key);
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        pcmgxxktempService.removeByPcmgxmlid(idList);
        removeByIds(idList);
    }

    @Override
    @Transactional
    public PcmGxml get(String key) {
        PcmGxml et = getById(key);
        if(et == null){
            et = new PcmGxml();
            et.setPcmgxmlid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public PcmGxml getDraft(PcmGxml et) {
        return et;
    }

    @Override
    public boolean checkKey(PcmGxml et) {
        return (!ObjectUtils.isEmpty(et.getPcmgxmlid())) && (!Objects.isNull(this.getById(et.getPcmgxmlid())));
    }
    @Override
    @Transactional
    public boolean save(PcmGxml et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(PcmGxml et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<PcmGxml> list) {
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<PcmGxml> list) {
        saveOrUpdateBatch(list,batchSize);
    }



    /**
     * 查询集合 当前年度高校名录
     */
    @Override
    public Page<PcmGxml> searchCurND(PcmGxmlSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmGxml> pages=baseMapper.searchCurND(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmGxml>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PcmGxml> searchDefault(PcmGxmlSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmGxml> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmGxml>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }







    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PcmGxml> getPcmgxmlByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PcmGxml> getPcmgxmlByEntities(List<PcmGxml> entities) {
        List ids =new ArrayList();
        for(PcmGxml entity : entities){
            Serializable id=entity.getPcmgxmlid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



