package cn.ibizlab.ehr.core.pcm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.pcm.domain.PcmProfileApproval;
import cn.ibizlab.ehr.core.pcm.filter.PcmProfileApprovalSearchContext;
import cn.ibizlab.ehr.core.pcm.service.IPcmProfileApprovalService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.pcm.mapper.PcmProfileApprovalMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[应聘者审批表] 服务对象接口实现
 */
@Slf4j
@Service("PcmProfileApprovalServiceImpl")
public class PcmProfileApprovalServiceImpl extends ServiceImpl<PcmProfileApprovalMapper, PcmProfileApproval> implements IPcmProfileApprovalService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pcm.service.IPcmProfileService pcmprofileService;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pim.service.IPimByzzjlmxService pimbyzzjlmxService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(PcmProfileApproval et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmprofileapprovalid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<PcmProfileApproval> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(PcmProfileApproval et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("pcmprofileapprovalid", et.getPcmprofileapprovalid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmprofileapprovalid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<PcmProfileApproval> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public PcmProfileApproval get(String key) {
        PcmProfileApproval et = getById(key);
        if(et == null){
            et = new PcmProfileApproval();
            et.setPcmprofileapprovalid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public PcmProfileApproval getDraft(PcmProfileApproval et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(PcmProfileApproval et) {
        return (!ObjectUtils.isEmpty(et.getPcmprofileapprovalid())) && (!Objects.isNull(this.getById(et.getPcmprofileapprovalid())));
    }
    @Override
    @Transactional
    public boolean save(PcmProfileApproval et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(PcmProfileApproval et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<PcmProfileApproval> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<PcmProfileApproval> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<PcmProfileApproval> selectByPcmprofileid(String pcmprofileid) {
        return baseMapper.selectByPcmprofileid(pcmprofileid);
    }
    @Override
    public void resetByPcmprofileid(String pcmprofileid) {
        this.update(new UpdateWrapper<PcmProfileApproval>().set("pcmprofileid",null).eq("pcmprofileid",pcmprofileid));
    }

    @Override
    public void resetByPcmprofileid(Collection<String> ids) {
        this.update(new UpdateWrapper<PcmProfileApproval>().set("pcmprofileid",null).in("pcmprofileid",ids));
    }

    @Override
    public void removeByPcmprofileid(String pcmprofileid) {
        this.remove(new QueryWrapper<PcmProfileApproval>().eq("pcmprofileid",pcmprofileid));
    }

	@Override
    public List<PcmProfileApproval> selectByPimbyzzjlmxid(String pimbyzzjlmxid) {
        return baseMapper.selectByPimbyzzjlmxid(pimbyzzjlmxid);
    }
    @Override
    public void removeByPimbyzzjlmxid(Collection<String> ids) {
        this.remove(new QueryWrapper<PcmProfileApproval>().in("pimbyzzjlmxid",ids));
    }

    @Override
    public void removeByPimbyzzjlmxid(String pimbyzzjlmxid) {
        this.remove(new QueryWrapper<PcmProfileApproval>().eq("pimbyzzjlmxid",pimbyzzjlmxid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PcmProfileApproval> searchDefault(PcmProfileApprovalSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmProfileApproval> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmProfileApproval>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 拒绝
     */
    @Override
    public Page<PcmProfileApproval> searchDisagree(PcmProfileApprovalSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmProfileApproval> pages=baseMapper.searchDisagree(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmProfileApproval>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(PcmProfileApproval et){
        //实体关系[DER1N_PCMPROFILEAPPROVAL_PCMPROFILE_PCMPROFILEID]
        if(!ObjectUtils.isEmpty(et.getPcmprofileid())){
            cn.ibizlab.ehr.core.pcm.domain.PcmProfile pcmprofile=et.getPcmprofile();
            if(ObjectUtils.isEmpty(pcmprofile)){
                cn.ibizlab.ehr.core.pcm.domain.PcmProfile majorEntity=pcmprofileService.get(et.getPcmprofileid());
                et.setPcmprofile(majorEntity);
                pcmprofile=majorEntity;
            }
            et.setPcmprofilename(pcmprofile.getPcmprofilename());
        }
        //实体关系[DER1N_PCMPROFILEAPPROVAL_PIMBYZZJLMX_PIMBYZZJLMXID]
        if(!ObjectUtils.isEmpty(et.getPimbyzzjlmxid())){
            cn.ibizlab.ehr.core.pim.domain.PimByzzjlmx pimbyzzjlmx=et.getPimbyzzjlmx();
            if(ObjectUtils.isEmpty(pimbyzzjlmx)){
                cn.ibizlab.ehr.core.pim.domain.PimByzzjlmx majorEntity=pimbyzzjlmxService.get(et.getPimbyzzjlmxid());
                et.setPimbyzzjlmx(majorEntity);
                pimbyzzjlmx=majorEntity;
            }
            et.setPimbyzzjlmxname(pimbyzzjlmx.getPimbyzzjlmxname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PcmProfileApproval> getPcmprofileapprovalByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PcmProfileApproval> getPcmprofileapprovalByEntities(List<PcmProfileApproval> entities) {
        List ids =new ArrayList();
        for(PcmProfileApproval entity : entities){
            Serializable id=entity.getPcmprofileapprovalid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



