package cn.ibizlab.ehr.core.pcm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.pcm.domain.PcmWorkResume;
import cn.ibizlab.ehr.core.pcm.filter.PcmWorkResumeSearchContext;
import cn.ibizlab.ehr.core.pcm.service.IPcmWorkResumeService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.pcm.mapper.PcmWorkResumeMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[工作履历] 服务对象接口实现
 */
@Slf4j
@Service("PcmWorkResumeServiceImpl")
public class PcmWorkResumeServiceImpl extends ServiceImpl<PcmWorkResumeMapper, PcmWorkResume> implements IPcmWorkResumeService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pcm.service.IPcmProfileService pcmprofileService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(PcmWorkResume et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmworkresumeid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<PcmWorkResume> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(PcmWorkResume et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("pcmworkresumeid", et.getPcmworkresumeid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmworkresumeid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<PcmWorkResume> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public PcmWorkResume get(String key) {
        PcmWorkResume et = getById(key);
        if(et == null){
            et = new PcmWorkResume();
            et.setPcmworkresumeid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public PcmWorkResume getDraft(PcmWorkResume et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(PcmWorkResume et) {
        return (!ObjectUtils.isEmpty(et.getPcmworkresumeid())) && (!Objects.isNull(this.getById(et.getPcmworkresumeid())));
    }
    @Override
    @Transactional
    public boolean save(PcmWorkResume et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(PcmWorkResume et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<PcmWorkResume> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<PcmWorkResume> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<PcmWorkResume> selectByPcmprofileid(String pcmprofileid) {
        return baseMapper.selectByPcmprofileid(pcmprofileid);
    }
    @Override
    public void resetByPcmprofileid(String pcmprofileid) {
        this.update(new UpdateWrapper<PcmWorkResume>().set("pcmprofileid",null).eq("pcmprofileid",pcmprofileid));
    }

    @Override
    public void resetByPcmprofileid(Collection<String> ids) {
        this.update(new UpdateWrapper<PcmWorkResume>().set("pcmprofileid",null).in("pcmprofileid",ids));
    }

    @Override
    public void removeByPcmprofileid(String pcmprofileid) {
        this.remove(new QueryWrapper<PcmWorkResume>().eq("pcmprofileid",pcmprofileid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PcmWorkResume> searchDefault(PcmWorkResumeSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmWorkResume> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmWorkResume>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(PcmWorkResume et){
        //实体关系[DER1N_PCMWORKRESUME_PCMPROFILE_PCMPROFILEID]
        if(!ObjectUtils.isEmpty(et.getPcmprofileid())){
            cn.ibizlab.ehr.core.pcm.domain.PcmProfile pcmprofile=et.getPcmprofile();
            if(ObjectUtils.isEmpty(pcmprofile)){
                cn.ibizlab.ehr.core.pcm.domain.PcmProfile majorEntity=pcmprofileService.get(et.getPcmprofileid());
                et.setPcmprofile(majorEntity);
                pcmprofile=majorEntity;
            }
            et.setPcmprofilename(pcmprofile.getPcmprofilename());
            et.setCertificatenumber(pcmprofile.getCertificatenumber());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PcmWorkResume> getPcmworkresumeByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PcmWorkResume> getPcmworkresumeByEntities(List<PcmWorkResume> entities) {
        List ids =new ArrayList();
        for(PcmWorkResume entity : entities){
            Serializable id=entity.getPcmworkresumeid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



