package cn.ibizlab.ehr.core.pcm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.pcm.domain.PcmYddgmx;
import cn.ibizlab.ehr.core.pcm.filter.PcmYddgmxSearchContext;
import cn.ibizlab.ehr.core.pcm.service.IPcmYddgmxService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.pcm.mapper.PcmYddgmxMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[异动待岗明细] 服务对象接口实现
 */
@Slf4j
@Service("PcmYddgmxServiceImpl")
public class PcmYddgmxServiceImpl extends ServiceImpl<PcmYddgmxMapper, PcmYddgmx> implements IPcmYddgmxService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmOrgsectorService ormorgsectorService;

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pcm.service.logic.IPcmYddgmxSetFinishedLogic setfinishedLogic;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(PcmYddgmx et) {
        fillParentData(et);
        createIndexMajorEntityData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmyddgmxid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<PcmYddgmx> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(PcmYddgmx et) {
        fillParentData(et);
        setfinishedLogic.execute(et);
        pcmydmxService.update(pcmyddgmxInheritMapping.toPcmydmx(et));
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("pcmyddgmxid", et.getPcmyddgmxid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmyddgmxid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<PcmYddgmx> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        pcmydmxService.remove(key);
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public PcmYddgmx get(String key) {
        PcmYddgmx et = getById(key);
        if(et == null){
            et = new PcmYddgmx();
            et.setPcmyddgmxid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public PcmYddgmx getDraft(PcmYddgmx et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(PcmYddgmx et) {
        return (!ObjectUtils.isEmpty(et.getPcmyddgmxid())) && (!Objects.isNull(this.getById(et.getPcmyddgmxid())));
    }
    @Override
    @Transactional
    public PcmYddgmx fillPersonInfo(PcmYddgmx et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public PcmYddgmx qXDG(PcmYddgmx et) {
        //自定义代码
        return et;
    }

    @Override
    @Transactional
    public boolean save(PcmYddgmx et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(PcmYddgmx et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<PcmYddgmx> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<PcmYddgmx> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<PcmYddgmx> selectByOrmorgsectorid(String orgsectorid) {
        return baseMapper.selectByOrmorgsectorid(orgsectorid);
    }
    @Override
    public void removeByOrmorgsectorid(String orgsectorid) {
        this.remove(new QueryWrapper<PcmYddgmx>().eq("ormorgsectorid",orgsectorid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PcmYddgmx> searchDefault(PcmYddgmxSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmYddgmx> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmYddgmx>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 管理查询
     */
    @Override
    public Page<PcmYddgmx> searchGLDS(PcmYddgmxSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmYddgmx> pages=baseMapper.searchGLDS(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmYddgmx>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 记录查询
     */
    @Override
    public Page<PcmYddgmx> searchJLDS(PcmYddgmxSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmYddgmx> pages=baseMapper.searchJLDS(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmYddgmx>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(PcmYddgmx et){
        //实体关系[DER1N_PCMYDDGMX_ORMORGSECTOR_ORMORGSECTORID]
        if(!ObjectUtils.isEmpty(et.getOrmorgsectorid())){
            cn.ibizlab.ehr.core.orm.domain.OrmOrgsector ormorgsector=et.getOrmorgsector();
            if(ObjectUtils.isEmpty(ormorgsector)){
                cn.ibizlab.ehr.core.orm.domain.OrmOrgsector majorEntity=ormorgsectorService.get(et.getOrmorgsectorid());
                et.setOrmorgsector(majorEntity);
                ormorgsector=majorEntity;
            }
            et.setOrmorgsectorname(ormorgsector.getShortname());
            et.setDgshortname(ormorgsector.getShortname());
        }
    }



    @Autowired
    cn.ibizlab.ehr.core.pcm.mapping.PcmYddgmxInheritMapping pcmyddgmxInheritMapping;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pcm.service.IPcmYdmxService pcmydmxService;

    /**
     * 创建索引主实体数据
     * @param et
     */
    private void createIndexMajorEntityData(PcmYddgmx et){
        if(ObjectUtils.isEmpty(et.getPcmyddgmxid()))
            et.setPcmyddgmxid((String)et.getDefaultKey(true));
        cn.ibizlab.ehr.core.pcm.domain.PcmYdmx pcmydmx =pcmyddgmxInheritMapping.toPcmydmx(et);
        pcmydmx.set("pcmydmxtype","50");
        pcmydmxService.create(pcmydmx);
    }

    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PcmYddgmx> getPcmyddgmxByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PcmYddgmx> getPcmyddgmxByEntities(List<PcmYddgmx> entities) {
        List ids =new ArrayList();
        for(PcmYddgmx entity : entities){
            Serializable id=entity.getPcmyddgmxid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



