package cn.ibizlab.ehr.core.pcm.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.pcm.domain.PcmZjdyzw;
import cn.ibizlab.ehr.core.pcm.filter.PcmZjdyzwSearchContext;
import cn.ibizlab.ehr.core.pcm.service.IPcmZjdyzwService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.pcm.mapper.PcmZjdyzwMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[职级和职务对应关系] 服务对象接口实现
 */
@Slf4j
@Service("PcmZjdyzwServiceImpl")
public class PcmZjdyzwServiceImpl extends ServiceImpl<PcmZjdyzwMapper, PcmZjdyzw> implements IPcmZjdyzwService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmDutyService ormdutyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmRankService ormrankService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(PcmZjdyzw et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmzjdyzwid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<PcmZjdyzw> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(PcmZjdyzw et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("pcmzjdyzwid", et.getPcmzjdyzwid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPcmzjdyzwid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<PcmZjdyzw> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public PcmZjdyzw get(String key) {
        PcmZjdyzw et = getById(key);
        if(et == null){
            et = new PcmZjdyzw();
            et.setPcmzjdyzwid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public PcmZjdyzw getDraft(PcmZjdyzw et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(PcmZjdyzw et) {
        return (!ObjectUtils.isEmpty(et.getPcmzjdyzwid())) && (!Objects.isNull(this.getById(et.getPcmzjdyzwid())));
    }
    @Override
    @Transactional
    public boolean save(PcmZjdyzw et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(PcmZjdyzw et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<PcmZjdyzw> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<PcmZjdyzw> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<PcmZjdyzw> selectByOrmdutyid(String ormdutyid) {
        return baseMapper.selectByOrmdutyid(ormdutyid);
    }
    @Override
    public void resetByOrmdutyid(String ormdutyid) {
        this.update(new UpdateWrapper<PcmZjdyzw>().set("ormdutyid",null).eq("ormdutyid",ormdutyid));
    }

    @Override
    public void resetByOrmdutyid(Collection<String> ids) {
        this.update(new UpdateWrapper<PcmZjdyzw>().set("ormdutyid",null).in("ormdutyid",ids));
    }

    @Override
    public void removeByOrmdutyid(String ormdutyid) {
        this.remove(new QueryWrapper<PcmZjdyzw>().eq("ormdutyid",ormdutyid));
    }

	@Override
    public List<PcmZjdyzw> selectByOrmrankid(String ormrankid) {
        return baseMapper.selectByOrmrankid(ormrankid);
    }
    @Override
    public void resetByOrmrankid(String ormrankid) {
        this.update(new UpdateWrapper<PcmZjdyzw>().set("ormrankid",null).eq("ormrankid",ormrankid));
    }

    @Override
    public void resetByOrmrankid(Collection<String> ids) {
        this.update(new UpdateWrapper<PcmZjdyzw>().set("ormrankid",null).in("ormrankid",ids));
    }

    @Override
    public void removeByOrmrankid(String ormrankid) {
        this.remove(new QueryWrapper<PcmZjdyzw>().eq("ormrankid",ormrankid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PcmZjdyzw> searchDefault(PcmZjdyzwSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PcmZjdyzw> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PcmZjdyzw>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(PcmZjdyzw et){
        //实体关系[DER1N_PCMZJDYZW_ORMDUTY_ORMDUTYID]
        if(!ObjectUtils.isEmpty(et.getOrmdutyid())){
            cn.ibizlab.ehr.core.orm.domain.OrmDuty ormduty=et.getOrmduty();
            if(ObjectUtils.isEmpty(ormduty)){
                cn.ibizlab.ehr.core.orm.domain.OrmDuty majorEntity=ormdutyService.get(et.getOrmdutyid());
                et.setOrmduty(majorEntity);
                ormduty=majorEntity;
            }
            et.setOrmdutyname(ormduty.getOrmdutyname());
        }
        //实体关系[DER1N_PCMZJDYZW_ORMRANK_ORMRANKID]
        if(!ObjectUtils.isEmpty(et.getOrmrankid())){
            cn.ibizlab.ehr.core.orm.domain.OrmRank ormrank=et.getOrmrank();
            if(ObjectUtils.isEmpty(ormrank)){
                cn.ibizlab.ehr.core.orm.domain.OrmRank majorEntity=ormrankService.get(et.getOrmrankid());
                et.setOrmrank(majorEntity);
                ormrank=majorEntity;
            }
            et.setOrmrankname(ormrank.getOrmrankname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PcmZjdyzw> getPcmzjdyzwByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PcmZjdyzw> getPcmzjdyzwByEntities(List<PcmZjdyzw> entities) {
        List ids =new ArrayList();
        for(PcmZjdyzw entity : entities){
            Serializable id=entity.getPcmzjdyzwid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



