package cn.ibizlab.ehr.core.pim.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.pim.domain.PimTitleCatalogue;
import cn.ibizlab.ehr.core.pim.filter.PimTitleCatalogueSearchContext;
import cn.ibizlab.ehr.core.pim.service.IPimTitleCatalogueService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.pim.mapper.PimTitleCatalogueMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[职称目录] 服务对象接口实现
 */
@Slf4j
@Service("PimTitleCatalogueServiceImpl")
public class PimTitleCatalogueServiceImpl extends ServiceImpl<PimTitleCatalogueMapper, PimTitleCatalogue> implements IPimTitleCatalogueService {


    protected cn.ibizlab.ehr.core.pim.service.IPimTitleCatalogueService pimtitlecatalogueService = this;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.pim.service.IPimTitleService pimtitleService;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmOrgService ormorgService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(PimTitleCatalogue et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPimtitlecatalogueid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<PimTitleCatalogue> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(PimTitleCatalogue et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("pimtitlecatalogueid", et.getPimtitlecatalogueid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getPimtitlecatalogueid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<PimTitleCatalogue> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public PimTitleCatalogue get(String key) {
        PimTitleCatalogue et = getById(key);
        if(et == null){
            et = new PimTitleCatalogue();
            et.setPimtitlecatalogueid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public PimTitleCatalogue getDraft(PimTitleCatalogue et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(PimTitleCatalogue et) {
        return (!ObjectUtils.isEmpty(et.getPimtitlecatalogueid())) && (!Objects.isNull(this.getById(et.getPimtitlecatalogueid())));
    }
    @Override
    @Transactional
    public boolean save(PimTitleCatalogue et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(PimTitleCatalogue et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<PimTitleCatalogue> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<PimTitleCatalogue> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<PimTitleCatalogue> selectByOrmorgid(String orgid) {
        return baseMapper.selectByOrmorgid(orgid);
    }
    @Override
    public void removeByOrmorgid(String orgid) {
        this.remove(new QueryWrapper<PimTitleCatalogue>().eq("ormorgid",orgid));
    }

	@Override
    public List<PimTitleCatalogue> selectByPimtitlecatalogueid2(String pimtitlecatalogueid) {
        return baseMapper.selectByPimtitlecatalogueid2(pimtitlecatalogueid);
    }
    @Override
    public void removeByPimtitlecatalogueid2(String pimtitlecatalogueid) {
        this.remove(new QueryWrapper<PimTitleCatalogue>().eq("pimtitlecatalogueid2",pimtitlecatalogueid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<PimTitleCatalogue> searchDefault(PimTitleCatalogueSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PimTitleCatalogue> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PimTitleCatalogue>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 查询没有上级职称的
     */
    @Override
    public Page<PimTitleCatalogue> searchIsRootDQ(PimTitleCatalogueSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PimTitleCatalogue> pages=baseMapper.searchIsRootDQ(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PimTitleCatalogue>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 不查询没有上级职称的
     */
    @Override
    public Page<PimTitleCatalogue> searchNotRootDQ(PimTitleCatalogueSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<PimTitleCatalogue> pages=baseMapper.searchNotRootDQ(context.getPages(),context,context.getSelectCond());
        return new PageImpl<PimTitleCatalogue>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(PimTitleCatalogue et){
        //实体关系[DER1N_PIMTITLECATALOGUE_PIMTITLECATALOGUE_PIMTITLECATALOGUEID2]
        if(!ObjectUtils.isEmpty(et.getPimtitlecatalogueid2())){
            cn.ibizlab.ehr.core.pim.domain.PimTitleCatalogue pimtitlecatalogue2=et.getPimtitlecatalogue2();
            if(ObjectUtils.isEmpty(pimtitlecatalogue2)){
                cn.ibizlab.ehr.core.pim.domain.PimTitleCatalogue majorEntity=pimtitlecatalogueService.get(et.getPimtitlecatalogueid2());
                et.setPimtitlecatalogue2(majorEntity);
                pimtitlecatalogue2=majorEntity;
            }
            et.setPimtitlecataloguename2(pimtitlecatalogue2.getPimtitlecataloguename());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<PimTitleCatalogue> getPimtitlecatalogueByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<PimTitleCatalogue> getPimtitlecatalogueByEntities(List<PimTitleCatalogue> entities) {
        List ids =new ArrayList();
        for(PimTitleCatalogue entity : entities){
            Serializable id=entity.getPimtitlecatalogueid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



