package cn.ibizlab.ehr.core.sal.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.beans.factory.annotation.Value;
import cn.ibizlab.ehr.util.errors.BadRequestAlertException;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.sal.domain.SalStdXmgz;
import cn.ibizlab.ehr.core.sal.filter.SalStdXmgzSearchContext;
import cn.ibizlab.ehr.core.sal.service.ISalStdXmgzService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;
import cn.ibizlab.ehr.util.helper.DEFieldCacheMap;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.sal.mapper.SalStdXmgzMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[项目工资标准] 服务对象接口实现
 */
@Slf4j
@Service("SalStdXmgzServiceImpl")
public class SalStdXmgzServiceImpl extends ServiceImpl<SalStdXmgzMapper, SalStdXmgz> implements ISalStdXmgzService {

    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmDutyService ormdutyService;
    @Autowired
    @Lazy
    protected cn.ibizlab.ehr.core.orm.service.IOrmOrgService ormorgService;

    protected int batchSize = 500;

    @Override
    @Transactional
    public boolean create(SalStdXmgz et) {
        fillParentData(et);
        if(!this.retBool(this.baseMapper.insert(et))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getSalstdxmgzid()), et);
        return true;
    }

    @Override
    @Transactional
    public void createBatch(List<SalStdXmgz> list) {
        list.forEach(item->fillParentData(item));
        this.saveBatch(list, batchSize);
    }

    @Override
    @Transactional
    public boolean update(SalStdXmgz et) {
        fillParentData(et);
        if(!update(et, (Wrapper) et.getUpdateWrapper(true).eq("salstdxmgzid", et.getSalstdxmgzid()))) {
            return false;
        }
        CachedBeanCopier.copy(get(et.getSalstdxmgzid()), et);
        return true;
    }

    @Override
    @Transactional
    public void updateBatch(List<SalStdXmgz> list) {
        list.forEach(item->fillParentData(item));
        updateBatchById(list, batchSize);
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result = removeById(key);
        return result ;
    }

    @Override
    @Transactional
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public SalStdXmgz get(String key) {
        SalStdXmgz et = getById(key);
        if(et == null){
            et = new SalStdXmgz();
            et.setSalstdxmgzid(key);
        }
        else {
        }
        return et;
    }

    @Override
    public SalStdXmgz getDraft(SalStdXmgz et) {
        fillParentData(et);
        return et;
    }

    @Override
    public boolean checkKey(SalStdXmgz et) {
        return (!ObjectUtils.isEmpty(et.getSalstdxmgzid())) && (!Objects.isNull(this.getById(et.getSalstdxmgzid())));
    }
    @Override
    @Transactional
    public boolean save(SalStdXmgz et) {
        if(!saveOrUpdate(et)) {
            return false;
        }
        return true;
    }

    @Override
    @Transactional
    public boolean saveOrUpdate(SalStdXmgz et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    @Transactional
    public boolean saveBatch(Collection<SalStdXmgz> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
        return true;
    }

    @Override
    @Transactional
    public void saveBatch(List<SalStdXmgz> list) {
        list.forEach(item->fillParentData(item));
        saveOrUpdateBatch(list,batchSize);
    }


	@Override
    public List<SalStdXmgz> selectByOrmdutyid(String ormdutyid) {
        return baseMapper.selectByOrmdutyid(ormdutyid);
    }
    @Override
    public void removeByOrmdutyid(String ormdutyid) {
        this.remove(new QueryWrapper<SalStdXmgz>().eq("ormdutyid",ormdutyid));
    }

	@Override
    public List<SalStdXmgz> selectByOrmorgid(String orgid) {
        return baseMapper.selectByOrmorgid(orgid);
    }
    @Override
    public void removeByOrmorgid(String orgid) {
        this.remove(new QueryWrapper<SalStdXmgz>().eq("ormorgid",orgid));
    }


    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<SalStdXmgz> searchDefault(SalStdXmgzSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<SalStdXmgz> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<SalStdXmgz>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }



    /**
     * 为当前实体填充父数据（外键值文本、外键值附加数据）
     * @param et
     */
    private void fillParentData(SalStdXmgz et){
        //实体关系[DER1N_SALSTDXMGZ_ORMDUTY_ORMDUTYID]
        if(!ObjectUtils.isEmpty(et.getOrmdutyid())){
            cn.ibizlab.ehr.core.orm.domain.OrmDuty ormduty=et.getOrmduty();
            if(ObjectUtils.isEmpty(ormduty)){
                cn.ibizlab.ehr.core.orm.domain.OrmDuty majorEntity=ormdutyService.get(et.getOrmdutyid());
                et.setOrmduty(majorEntity);
                ormduty=majorEntity;
            }
            et.setOrmdutyname(ormduty.getOrmdutyname());
        }
        //实体关系[DER1N_SALSTDXMGZ_ORMORG_ORMORGID]
        if(!ObjectUtils.isEmpty(et.getOrmorgid())){
            cn.ibizlab.ehr.core.orm.domain.OrmOrg ormorg=et.getOrmorg();
            if(ObjectUtils.isEmpty(ormorg)){
                cn.ibizlab.ehr.core.orm.domain.OrmOrg majorEntity=ormorgService.get(et.getOrmorgid());
                et.setOrmorg(majorEntity);
                ormorg=majorEntity;
            }
            et.setOrmorgname(ormorg.getOrgname());
        }
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

    @Override
    public List<SalStdXmgz> getSalstdxmgzByIds(List<String> ids) {
         return this.listByIds(ids);
    }

    @Override
    public List<SalStdXmgz> getSalstdxmgzByEntities(List<SalStdXmgz> entities) {
        List ids =new ArrayList();
        for(SalStdXmgz entity : entities){
            Serializable id=entity.getSalstdxmgzid();
            if(!ObjectUtils.isEmpty(id)){
                ids.add(id);
            }
        }
        if(ids.size()>0) {
            return this.listByIds(ids);
        }
        else {
            return entities;
        }
    }




}



