package cn.ibizlab.ehr.core.wf.service.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Map;
import java.util.HashSet;
import java.util.HashMap;
import java.util.Collection;
import java.util.Objects;
import java.util.Optional;
import java.math.BigInteger;

import lombok.extern.slf4j.Slf4j;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cglib.beans.BeanCopier;
import org.springframework.stereotype.Service;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.util.ObjectUtils;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.context.annotation.Lazy;
import cn.ibizlab.ehr.core.wf.domain.WFActor;
import cn.ibizlab.ehr.core.wf.filter.WFActorSearchContext;
import cn.ibizlab.ehr.core.wf.service.IWFActorService;

import cn.ibizlab.ehr.util.helper.CachedBeanCopier;


import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import cn.ibizlab.ehr.core.wf.mapper.WFActorMapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.alibaba.fastjson.JSONObject;
import org.springframework.util.StringUtils;

/**
 * 实体[工作流操作者] 服务对象接口实现
 */
@Slf4j
@Service("WFActorServiceImpl")
public class WFActorServiceImpl extends ServiceImpl<WFActorMapper, WFActor> implements IWFActorService {

    @Autowired
    @Lazy
    private cn.ibizlab.ehr.core.wf.service.IWFTmpStepActorService wftmpstepactorService;

    private int batchSize = 500;

    @Override
    @Transactional
    public boolean update(WFActor et) {
        if(!update(et,(Wrapper) et.getUpdateWrapper(true).eq("wfactorid",et.getWfactorid())))
            return false;
        CachedBeanCopier.copy(get(et.getWfactorid()),et);
        return true;
    }

    @Override
    public void updateBatch(List<WFActor> list) {
        updateBatchById(list,batchSize);
    }

    @Override
    public boolean checkKey(WFActor et) {
        return (!ObjectUtils.isEmpty(et.getWfactorid()))&&(!Objects.isNull(this.getById(et.getWfactorid())));
    }

    @Override
    @Transactional
    public WFActor get(String key) {
        WFActor et = getById(key);
        if(et==null){
            et=new WFActor();
            et.setWfactorid(key);
        }
        else{
        }
        return et;
    }

    @Override
    @Transactional
    public boolean remove(String key) {
        boolean result=removeById(key);
        return result ;
    }

    @Override
    public void removeBatch(Collection<String> idList) {
        removeByIds(idList);
    }

    @Override
    @Transactional
    public boolean create(WFActor et) {
        if(!this.retBool(this.baseMapper.insert(et)))
            return false;
        CachedBeanCopier.copy(get(et.getWfactorid()),et);
        return true;
    }

    @Override
    public void createBatch(List<WFActor> list) {
        saveBatch(list,batchSize);
    }

    @Override
    @Transactional
    public boolean save(WFActor et) {
        if(!saveOrUpdate(et))
            return false;
        return true;
    }

    @Override
    @Transactional(
            rollbackFor = {Exception.class}
    )
    public boolean saveOrUpdate(WFActor et) {
        if (null == et) {
            return false;
        } else {
            return checkKey(et) ? this.update(et) : this.create(et);
        }
    }

    @Override
    public void saveBatch(List<WFActor> list) {
        saveOrUpdateBatch(list,batchSize);
    }

    @Override
    public WFActor getDraft(WFActor et) {
        return et;
    }



    /**
     * 查询集合 IndexDER
     */
    @Override
    public Page<WFActor> searchIndexDER(WFActorSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<WFActor> pages=baseMapper.searchIndexDER(context.getPages(),context,context.getSelectCond());
        return new PageImpl<WFActor>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }

    /**
     * 查询集合 DEFAULT
     */
    @Override
    public Page<WFActor> searchDefault(WFActorSearchContext context) {
        com.baomidou.mybatisplus.extension.plugins.pagination.Page<WFActor> pages=baseMapper.searchDefault(context.getPages(),context,context.getSelectCond());
        return new PageImpl<WFActor>(pages.getRecords(), context.getPageable(), pages.getTotal());
    }




    @Override
    public List<JSONObject> select(String sql, Map param){
        return this.baseMapper.selectBySQL(sql,param);
    }

    @Override
    @Transactional
    public boolean execute(String sql , Map param){
        if (sql == null || sql.isEmpty()) {
            return false;
        }
        if (sql.toLowerCase().trim().startsWith("insert")) {
            return this.baseMapper.insertBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("update")) {
            return this.baseMapper.updateBySQL(sql,param);
        }
        if (sql.toLowerCase().trim().startsWith("delete")) {
            return this.baseMapper.deleteBySQL(sql,param);
        }
        log.warn("暂未支持的SQL语法");
        return true;
    }

}


